@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: diff.cc 1.20.1.19.1.3.1.6.1.12 Sun, 06 Jul 1997 01:37:54 -0700 jmacd $
+ * $Id: diff.cc 1.20.1.19.1.3.1.6.1.19 Wed, 14 Oct 1998 20:50:34 -0700 jmacd $
  */
 
 #include "prcs.h"
@@ -41,14 +41,15 @@ static PrPrcsExitStatusError diff_named_versions();
 static PrBoolError diff_similar_files(ProjectDescriptor* toP, ProjectDescriptor* fromP);
 static PrBoolError diff_missing_files(ProjectDescriptor* toP, ProjectDescriptor* fromP);
 static PrBoolError diff_symlink_pair(FileEntry*, FileEntry*);
-static PrBoolError diff_new_file(FileEntry*);
+static PrBoolError diff_new_file(FileEntry*, ProjectDescriptor* fromP);
 static PrVoidError prepare_diff_workingfile(FileEntry *fe);
 static PrIntError try_optimizations(FileEntry *tofe, FileEntry *fromfe);
 static PrVoidError prepare_diff_versionfile(FileEntry *fe, bool use_working);
 static PrBoolError diff_project_files(ProjectDescriptor* toP, ProjectDescriptor* fromP);
 static PrBoolError diff_two_files(const char* label1, const char* label2,
 				  const char* file1, const char* file2,
-				  const char* index, SystemCommand *diff_command);
+				  const char* index, const char* from_project,
+				  SystemCommand *diff_command);
 static PrVoidError get_info(FileEntry* fe, RepEntry* rep_entry);
 
 static bool use_working_to_file = true;
@@ -86,7 +87,7 @@ static PrPrcsExitStatusError diff_working_files()
     use_working_to_file = true;
 
     if(!fs_file_readable(cmd_root_project_file_path))
-	throw prcserror << "Can't open file " << squote(cmd_root_project_file_path) << perror;
+	pthrow prcserror << "Can't open file " << squote(cmd_root_project_file_path) << perror;
 
     /* Set up the project file diff */
     const char* abs_path;
@@ -94,9 +95,9 @@ static PrPrcsExitStatusError diff_working_files()
     Return_if_fail(abs_path << absolute_path(cmd_root_project_file_path));
 
     If_fail(Err_symlink(abs_path, temp_file_to))
-	throw prcserror << "Failed creating symlink " << squote(temp_file_to) << perror;
+	pthrow prcserror << "Failed creating symlink " << squote(temp_file_to) << perror;
 
-    Return_if_fail(to << read_project_file(cmd_root_project_full_name, temp_file_to, true, KeepKeywords));
+    Return_if_fail(to << read_project_file(cmd_root_project_full_name, temp_file_to, true, KeepNothing));
 
     Return_if_fail(to->init_repository_entry(cmd_root_project_name, false, false));
 
@@ -113,13 +114,13 @@ static PrPrcsExitStatusError diff_working_files()
 	Return_if_fail(from << checkout_create_empty_prj_file(temp_file_from,
 							      cmd_root_project_full_name,
 							      from_version->prcs_major(),
-							      KeepKeywords));
+							      KeepNothing));
     } else {
 	Return_if_fail(from << to->repository_entry() ->
 		       checkout_create_prj_file(temp_file_from,
 						cmd_root_project_full_name,
 						from_version->rcs_version(),
-						KeepKeywords));
+						KeepNothing));
     }
 
     eliminate_unnamed_files(from);
@@ -181,13 +182,13 @@ static PrPrcsExitStatusError diff_named_versions()
 	Return_if_fail(from << checkout_create_empty_prj_file(temp_file_from,
 							      cmd_root_project_full_name,
 							      from_version->prcs_major(),
-							      KeepKeywords));
+							      KeepNothing));
     } else {
 	Return_if_fail(from << rep_entry ->
 		       checkout_create_prj_file(temp_file_from,
 						cmd_root_project_full_name,
 						from_version->rcs_version(),
-						KeepKeywords));
+						KeepNothing));
     }
 
     eliminate_unnamed_files(from);
@@ -204,13 +205,13 @@ static PrPrcsExitStatusError diff_named_versions()
 	Return_if_fail(to << checkout_create_empty_prj_file(temp_file_to,
 							    cmd_root_project_full_name,
 							    to_version->prcs_major(),
-							    KeepKeywords));
+							    KeepNothing));
     } else {
 	Return_if_fail(to << rep_entry ->
 		       checkout_create_prj_file(temp_file_to,
 						cmd_root_project_full_name,
 						to_version->rcs_version(),
-						KeepKeywords));
+						KeepNothing));
     }
 
     eliminate_unnamed_files(to);
@@ -261,7 +262,7 @@ static PrVoidError get_info(FileEntry* fe, RepEntry* rep_entry)
 static PrVoidError prepare_diff_workingfile(FileEntry *fe)
 {
     If_fail(Err_unlink(temp_file_to)) {
-	throw prcserror << "Unlink " << squote(temp_file_to) << " failed" << perror;
+	pthrow prcserror << "Unlink " << squote(temp_file_to) << " failed" << perror;
     }
 
     if (fe->file_type() == RealFile) {
@@ -279,7 +280,7 @@ static PrVoidError prepare_diff_workingfile(FileEntry *fe)
 	    Return_if_fail(abs_path << absolute_path(fe->working_path()));
 
 	    If_fail (Err_symlink(abs_path, temp_file_to)) {
-		throw prcserror << "Error creating symbolic link"
+		pthrow prcserror << "Error creating symbolic link"
 				<< squote(temp_file_to) << perror;
 	    }
 	}
@@ -348,7 +349,7 @@ static PrIntError try_optimizations(FileEntry *tofe, FileEntry *fromfe)
 					  Unsetkeys));
 	} else {
 	    If_fail(setkey_buf.map_file(tofe->working_path()))
-		throw prcserror << "Read failure on " << squote(tofe->working_path())
+		pthrow prcserror << "Read failure on " << squote(tofe->working_path())
 				<< perror;
 	}
 
@@ -517,7 +518,7 @@ static PrBoolError diff_missing_files(ProjectDescriptor* toP,
 		else
 		    Return_if_fail(prepare_diff_versionfile(fe, false));
 
-		diff_new_file (fe);
+		diff_new_file (fe, fromP);
 	    } else {
 		prcsoutput << "Only in " << toP->full_version() << ": " << fe->working_path() << prcsendl;
 	    }
@@ -549,7 +550,12 @@ static PrBoolError diff_symlink_pair(FileEntry* to, FileEntry* from)
     const char *link1, *link2;
 
     link1 = from->link_name();
-    link2 = to->link_name();
+
+    if (use_working_to_file) {
+        Return_if_fail (link2 << read_sym_link (from->working_path()));
+    } else {
+        link2 = to->link_name();
+    }
 
     if(!pathname_equal(link1, link2)) {
 	prcsoutput << "Symlink " << diff_tuple(from, to)
@@ -561,7 +567,7 @@ static PrBoolError diff_symlink_pair(FileEntry* to, FileEntry* from)
     }
 }
 
-static PrBoolError diff_new_file(FileEntry *to)
+static PrBoolError diff_new_file(FileEntry *to, ProjectDescriptor* fromP)
 {
     Dstring label1, label2;
     Dstring filedes;
@@ -576,12 +582,14 @@ static PrBoolError diff_new_file(FileEntry *to)
 			to->file_mode());
     }
 
-    label1.sprintf("-L%s.orig %s %s ()",
+    label1.sprintf("-L%s/%s %s %s ()",
+		   fromP->full_version(),
 		   to->working_path(),
 		   get_utc_time(),
 		   get_login());
 
-    label2.sprintf("-L%s      %s %s %s",
+    label2.sprintf("-L%s/%s %s %s %s",
+		   to->project()->full_version(),
 		   to->working_path(),
 		   to->last_mod_date(),
 		   to->last_mod_user(),
@@ -592,7 +600,7 @@ static PrBoolError diff_new_file(FileEntry *to)
     if (to->file_attrs()->diff_tool())
 	cmd = sys_cmd_by_name (to->file_attrs()->diff_tool());
 
-    return diff_two_files(label1, label2, "/dev/null", temp_file_to, to->working_path(), cmd);
+    return diff_two_files(label1, label2, "/dev/null", temp_file_to, to->working_path(), fromP->full_version(), cmd);
 }
 
 void format_diff_label(FileEntry* fe, const char* name, Dstring* ds)
@@ -619,7 +627,7 @@ void format_diff_label(FileEntry* fe, const char* name, Dstring* ds)
 		     fe->file_mode());
     }
 
-    ds->sprintf("-L%s %s %s %s %s", name, date, user, desc.cast(), fe->project()->full_version());
+    ds->sprintf("-L%s/%s %s %s %s", fe->project()->full_version(), name, date, user, desc.cast());
 }
 
 PrBoolError diff_pair(FileEntry* from,
@@ -627,13 +635,10 @@ PrBoolError diff_pair(FileEntry* from,
 		      const char* from_file,
 		      const char* to_file)
 {
-    Dstring name1, name2, label1, label2;
+    Dstring label1, label2;
 
-    name1.sprintf("%s.orig", from->working_path());
-    name2.sprintf("%s     ", to->working_path());
-
-    format_diff_label(from, name1, &label1);
-    format_diff_label(to,   name2, &label2);
+    format_diff_label(from, from->working_path(), &label1);
+    format_diff_label(to,   to->working_path(), &label2);
 
     SystemCommand *cmd = NULL;
 
@@ -643,12 +648,13 @@ PrBoolError diff_pair(FileEntry* from,
     if (!cmd && from->file_attrs()->diff_tool())
 	cmd = sys_cmd_by_name (from->file_attrs()->diff_tool());
 
-    return diff_two_files(label1, label2, from_file, to_file, to->working_path(), cmd);
+    return diff_two_files(label1, label2, from_file, to_file, to->working_path(), from->project()->full_version(), cmd);
 }
 
 static PrBoolError diff_two_files(const char* label1, const char* label2,
 				  const char* file1, const char* file2,
-				  const char* index, SystemCommand *cmd)
+				  const char* index, const char* version,
+				  SystemCommand *cmd)
 {
     ArgList *args;
     char buf[512];
@@ -685,7 +691,13 @@ static PrBoolError diff_two_files(const char* label1, const char* label2,
 	output = cmd->standard_out();
     }
 
-    while((c = fread(buf, 1, 512, output)) > 0 ) {
+    for (;;) {
+        If_fail (c << Err_fread(buf, 512, output))
+	  pthrow prcserror << "Error reading from diff output pipe" << perror;
+
+	if (c == 0)
+	  break;
+
 	if(n_index_written) {
 	    /* this sorta depends on diffs output, a leading Files
 	     * means its in --brief mode, lets filter the filenames */
@@ -693,22 +705,20 @@ static PrBoolError diff_two_files(const char* label1, const char* label2,
 		prcsoutput << "The file " << squote(index) << " differs" << prcsendl;
 		break;
 	    }
-	    prcsoutput << "Index: " << index << prcsendl;
+	    prcsoutput << "Index: " << version << "/" << index << prcsendl;
 	    n_index_written = false;
 	}
-	fwrite(buf, 1, c, stdout);
+	If_fail (Err_fwrite(buf, c, stdout))
+	  pthrow prcserror << "Error writing diff output" << perror;
     }
 
-    if(ferror(output))
-	throw prcserror << "Error reading from diff output pipe" << perror;
-
     if (cmd)
 	Return_if_fail(ret << cmd->close());
     else
 	Return_if_fail(ret << gdiff_command.close());
 
     if(ret > 1)
-	throw prcserror << "Diff command exited abnormally on files "
+	pthrow prcserror << "Diff command exited abnormally on files "
 			 << squote(file1) << " and " << squote(file2) << dotendl;
 
     return (bool)ret;
@@ -718,13 +728,14 @@ static PrBoolError diff_project_files(ProjectDescriptor* toP, ProjectDescriptor*
 {
     Dstring label1, label2;
 
-    label1.sprintf("-L%s.orig %s", fromP->project_file_path(), fromP->full_version());
-    label2.sprintf("-L%s      %s", toP->project_file_path(), toP->full_version());
+    label1.sprintf("-L%s/%s", fromP->full_version(), fromP->project_file_path());
+    label2.sprintf("-L%s/%s", toP->full_version(), toP->project_file_path());
 
     return diff_two_files(label1,
 			  label2,
 			  temp_file_from,
 			  temp_file_to,
 			  toP->project_file_path(),
+			  fromP->full_version(),
 			  NULL /* @@@ */);
 }
