@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: prcserror.h 1.14.1.5.1.6 Tue, 20 May 1997 22:48:31 -0700 jmacd $
+ * $Id: prcserror.h 1.14.1.5.1.10 Tue, 06 Oct 1998 13:31:13 -0700 jmacd $
  */
 
 
@@ -26,8 +26,15 @@
 
 #include <iostream.h>
 #include <iomanip.h>
+#if defined(__GNUG__)
 #include <stdiostream.h>
 #include <strstream.h>
+#else
+#include <fstream.h>
+typedef filebuf stdiobuf;
+#include "be-strstream.h"
+#endif
+
 
 extern "C" {
 #include <stdio.h>
@@ -51,7 +58,7 @@ extern PrettyOstream prcserror;
 extern PrettyOstream prcsdebug;
 #endif
 
-#ifndef PRCS_DEVEL
+#if !defined(PRCS_DEVEL) || !defined(__GNUG__)
 extern ErrorToken global_error_token;
 extern int return_if_fail_if_ne_val;
 #endif
@@ -101,14 +108,14 @@ extern int return_if_fail_if_ne_val;
 class PrVoidError {
 public:
     PrVoidError(NonErrorToken) :_tok(FatalError), _error(false) { }
-#ifdef PRCS_DEVEL
+#if defined(PRCS_DEVEL) && defined(__GNUG__)
     explicit
 #endif
     PrVoidError(ErrorToken tok) :_tok(tok), _error(true) { }
     explicit PrVoidError(ostream& s);
 
     bool error() const {
-#ifndef PRCS_DEVEL
+#if !defined(PRCS_DEVEL) || !defined(__GNUG__)
 	global_error_token = _tok;
 #endif
 	return _error; }
@@ -121,10 +128,10 @@ protected:
 
 private:
     operator bool () const { return error(); }
-    operator!();
+    int operator!();
 };
 
-#ifdef PRCS_DEVEL
+#if defined(PRCS_DEVEL) && defined(__GNUG__)
 class NprVoidError {
 public:
     NprVoidError(NonErrorToken) :_error(false) { }
@@ -141,7 +148,7 @@ protected:
 
 private:
     operator bool () const { return error(); }
-    operator!();
+    int operator!();
 };
 #endif
 
@@ -173,7 +180,7 @@ public:
     PrError(const Type& val) :PrVoidError(NoError), _val(val) { }
     explicit PrError(ostream& s);
     PrError(PrVoidError err) :PrVoidError(err) { }
-#ifdef PRCS_DEVEL
+#if defined(PRCS_DEVEL) && defined(__GNUG__)
     explicit
 #endif
     PrError(ErrorToken tok) :PrVoidError(tok) { }
@@ -183,10 +190,10 @@ public:
 
 private:
     Type _val;
-    operator!();
+    int operator!();
 };
 
-#ifdef PRCS_DEVEL
+#if defined(PRCS_DEVEL) && defined(__GNUG__)
 template <class Type>
 class NprError : public NprVoidError {
 public:
@@ -199,7 +206,7 @@ public:
 
 private:
     Type _val;
-    operator!();
+    int operator!();
 };
 #endif
 
@@ -211,7 +218,7 @@ const PrError<Type>& operator<<(Type& var, const PrError<Type>& val)
     return val;
 }
 
-#ifdef PRCS_DEVEL
+#if defined(PRCS_DEVEL) && defined(__GNUG__)
 template <class Type>
 const NprError<Type>& operator<<(Type& var, const NprError<Type>& val)
 {
@@ -225,7 +232,7 @@ const NprError<Type>& operator<<(Type& var, const NprError<Type>& val)
 /*
  * if expr returns an error, return the same error
  */
-#ifdef PRCS_DEVEL
+#if defined(PRCS_DEVEL) && defined(__GNUG__)
 #define Return_if_fail(expr) \
      ({ typeof(expr) _E_(expr); if((_E_).error()) return _E_.void_error(); })
 #else
@@ -240,7 +247,7 @@ const NprError<Type>& operator<<(Type& var, const NprError<Type>& val)
  * if expr returns an error, return the same error, else, if
  * expr's value compares != to val, execute the following block
  */
-#ifdef PRCS_DEVEL
+#if defined(PRCS_DEVEL) && defined(__GNUG__)
 #define Return_if_fail_if_ne(expr, val) \
      if( ({ typeof(expr) _E_(expr); \
             typeof(_E_.non_error_val()) _E_Val_(_E_.non_error_val()); \
@@ -262,7 +269,7 @@ const NprError<Type>& operator<<(Type& var, const NprError<Type>& val)
 
 /* So that the error mechanism can be implemented with exception
  * handling later */
-#define throw return 0 <
+#define pthrow return 0 <
 
 /*********************************************************************/
 /*************************** Stream Types ****************************/
@@ -295,13 +302,16 @@ public:
     int set_column(int col);
     void reset_column();
 
-    virtual int xsputn(const char* s, int n);
-    int overflow(int c = EOF);
-    int sync();
-
     const char* fill_prefix() const;
     bool fill_pretty() const;
 
+#ifndef __GNUG__
+protected:
+#endif
+    virtual int xsputn(const char* s, int n);
+    virtual int overflow(int c = EOF);
+    virtual int sync();
+
 protected:
     streambuf* forward;
     int col, width;
@@ -320,9 +330,19 @@ protected:
 class PrettyOstream : public ostream {
 public:
     PrettyOstream(PrettyStreambuf* stream, ErrorToken err)
-	:ostream(stream), _buf(stream), _err(err) { }
+	:
+#ifndef __GNUG__
+         ios(stream),
+#endif
+         ostream(stream), _buf(stream), _err(err)
+        { }
     PrettyOstream(PrettyStreambuf* stream, NonErrorToken err)
-	:ostream(stream), _buf(stream), _err(err) { }
+        :
+#ifndef __GNUG__
+         ios(stream),
+#endif
+	 ostream(stream), _buf(stream), _err(err)
+        { }
     PrettyStreambuf& ostreambuf() const { return *_buf; }
     PrVoidError error() const { return _err; }
 protected:
@@ -441,6 +461,7 @@ public:
     ostream& option_manip(QueryOption);
     ostream& default_manip(QueryOption);
     ostream& force_manip(const char* message);
+    ostream& force_manip(const char* message, char different);
     ostream& report_manip(const char* message);
     ostream& query_manip(const char* message);
     ostream& string_query_manip(const char* message);
@@ -462,6 +483,7 @@ protected:
     QueryOption options[ MAX_QUERY_OPTIONS ];
 
     int default_option;
+    int force_option;
     const char* force_message;
     const char* report_message;
     const char* default_input;
@@ -472,7 +494,7 @@ protected:
 
 extern char const default_fail_query_message[];
 
-#ifndef __GNUG__
+#if !defined(__GNUG__) && !defined(__MWERKS__)
 template <class TP> class omanip {
     ostream& (*_f)(ostream&, TP);
     TP _a;
@@ -498,6 +520,15 @@ omanip<const char*> definput(const char* message);
 ostream& __omanip_force(ostream& s, const char* message);
 omanip<const char*> force(const char* message);
 
+struct CharAndStr {
+  CharAndStr (const char* str0, char c0) :str(str0), c(c0) { }
+  const char* str;
+  char c;
+};
+
+ostream& __omanip_force(ostream& s, CharAndStr cs);
+omanip<CharAndStr> force(const char* message, char different);
+
 ostream& __omanip_report(ostream& s, const char* message);
 omanip<const char*> report(const char* message);
 
