@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: misc.cc 1.30.1.3.1.17.1.11.1.7.1.18.1.23 Wed, 14 Jan 1998 23:22:44 -0800 jmacd $
+ * $Id: misc.cc 1.30.1.3.1.17.1.11.1.7.1.18.1.31 Wed, 14 Oct 1998 20:50:34 -0700 jmacd $
  */
 
 extern "C" {
@@ -75,6 +75,7 @@ int option_populate_delete = 0;
 int option_package_compress = 0;
 int option_be_silent = 0;
 int option_preserve_permissions = 0;
+int option_exclude_project_file = 0;
 int option_unlink = 0;
 int option_match_file = 0;
 int option_not_match_file = 0;
@@ -104,7 +105,7 @@ PrVoidError bug (void)
 {
   static const char maintainer[] = "prcs-bugs@XCF.Berkeley.EDU";
 
-  throw prcserror << "Please report this to " << maintainer << dotendl
+  pthrow prcserror << "Please report this to " << maintainer << dotendl
 		  << "When sending bug reports, always include:" << prcsendl
 		  << "-- a complete description of the problem encountered" << prcsendl
 		  << "-- the output of `prcs config'" << prcsendl
@@ -179,7 +180,7 @@ void protect_path(const char* path, Dstring* in)
     while(*path) {
         switch (*path) {
 	case '\"': case '\\': case ')':  case '(':  case ' ':
-	case '\t': case '\n': case '\v': case '\r': case '\f':
+	case '\t': case '\n': case '\v': case '\r': case '\f': case ';':
             in->append('\\');
 	default:
         in->append(*path);
@@ -195,7 +196,7 @@ void print_protected (const char* str, ostream& os)
 
         switch (c) {
 	case '\"': case '\\': case ')':  case '(':  case ' ':
-	case '\t': case '\n': case '\v': case '\r': case '\f':
+	case '\t': case '\n': case '\v': case '\r': case '\f': case ';':
             os << '\\';
 	default:
 	    os << c;
@@ -203,6 +204,19 @@ void print_protected (const char* str, ostream& os)
     }
 }
 
+void protect_string(const char* path, Dstring* in)
+{
+    while(*path) {
+        switch (*path) {
+	case '\"': case '\\':
+            in->append('\\');
+	default:
+        in->append(*path);
+	}
+        path += 1;
+    }
+}
+
 bool weird_pathname(const char* N)
 {
     /* don't allow anything outside the repository */
@@ -302,7 +316,7 @@ PrVoidError directory_recurse(const char* base,
     }
 
     if (! current_dir.OK() ) {
-	throw prcserror << "Error reading directory "
+	pthrow prcserror << "Error reading directory "
 			<< squote (base) << perror;
     }
 
@@ -331,7 +345,7 @@ PrVoidError directory_recurse_dirs(const char* base,
     }
 
     if (! current_dir.OK() ) {
-	throw prcserror << "Error reading directory "
+	pthrow prcserror << "Error reading directory "
 			<< squote (base) << perror;
     }
 
@@ -342,27 +356,31 @@ NprVoidError fs_nuke_file(const char* file)
 {
     bool ret = true;
     if(fs_is_directory(file)) {
-	const char* old_dir;
+	char old_dir[MAXPATHLEN];
+	const char* old_dir_name;
 
-	If_fail(old_dir << name_in_cwd(""))
+	If_fail(old_dir_name << name_in_cwd(""))
 	    ret = false;
 
-        If_fail(change_cwd(file)) {
+	strncpy (old_dir, old_dir_name, MAXPATHLEN-1);
+
+        If_fail(change_cwd(file))
             ret = false;
-        }
 
 	if (ret) {
-	    Dir current_dir(".");
+	    {
+		Dir current_dir(".");
 
-	    foreach(ent_ptr, current_dir, Dir::DirIterator) {
-                If_fail(fs_nuke_file(*ent_ptr)) {
-		    ret = false;
+		foreach(ent_ptr, current_dir, Dir::DirIterator) {
+		    If_fail(fs_nuke_file(*ent_ptr)) {
+			ret = false;
+		    }
 		}
+
+		Return_if_fail(change_cwd(old_dir));
+
+		ret &= current_dir.OK();
 	    }
-
-            Return_if_fail(change_cwd(old_dir));
-
-            ret &= current_dir.OK();
             ret &= rmdir(file) >= 0;
         }
 
@@ -553,14 +571,14 @@ PrConstCharPtrError name_in_cwd(const char* name)
            getcwd(buf, MAXPATHLEN)
 #endif
            == NULL) {
-            throw prcserror << "Getcwd failed" << perror;
+            pthrow prcserror << "Getcwd failed" << perror;
         } else
             changed = false;
         buflen = strlen(buf);
     }
 
     if(buflen + strlen(name) + 1 > MAXPATHLEN)
-        throw prcserror << "Failed building full pathname to " << squote(name)
+        pthrow prcserror << "Failed building full pathname to " << squote(name)
 			<< ", name too long" << dotendl;
 
     buf[buflen] = '/';
@@ -614,7 +632,7 @@ PrDstringPtrError guess_prj_name(const char* dir)
     }
 
     if (! current_dir.OK() ) {
-        throw prcserror << "Error reading current directory" << perror;
+        pthrow prcserror << "Error reading current directory" << perror;
     }
 
     if(found) {
@@ -633,13 +651,13 @@ PrConstCharPtrError read_sym_link(const char* name)
     int ret;
 
     if((ret = readlink(name, buf, MAXPATHLEN)) < 0) {
-        throw prcserror << "Error reading symlink " << squote(name) << perror;
+        pthrow prcserror << "Error reading symlink " << squote(name) << perror;
     } else {
         buf[ret] = '\0'; /* readlink() doesn't zero terminate */
     }
 
     if (ret == 0)
-	throw prcserror << "Symlink " << squote(name) << " may not be null" << dotendl;
+	pthrow prcserror << "Symlink " << squote(name) << " may not be null" << dotendl;
 
     return buf;
 }
@@ -665,10 +683,10 @@ PrConstCharPtrError show_file_info(const char* file)
     lsout->truncate(0);
 
     If_fail(read_string(out, lsout))
-	throw prcserror << "Read failure from ls output" << perror;
+	pthrow prcserror << "Read failure from ls output" << perror;
 
     Return_if_fail_if_ne(ls_command.close(), 0)
-        throw prcserror << ls_command.path() << " exited abnormally" << dotendl;
+        pthrow prcserror << ls_command.path() << " exited abnormally" << dotendl;
 
     return lsout->cast();
 }
@@ -696,7 +714,7 @@ PrConstCharPtrError absolute_path(const char* name)
     }
 }
 
-bool pathname_equal(const char*& a, const char*& b)
+bool pathname_equal(const char* const& a, const char* const& b)
 {
     const char *c(a), *d(b);
     for(;;) {
@@ -716,7 +734,7 @@ bool pathname_equal(const char*& a, const char*& b)
     }
 }
 
-int pathname_hash(const char*& s, int M)
+int pathname_hash(const char* const& s, int M)
 {
     const char *p;
     unsigned int h(0), g;
@@ -800,13 +818,13 @@ PrVoidError fs_write_filename(FILE *in, const char* filename)
     FILE* out;
 
     If_fail(out << Err_fopen(filename, "w"))
-	throw prcserror << "Failed opening file " << squote(filename)
+	pthrow prcserror << "Failed opening file " << squote(filename)
 		         << " for writing" << perror;
 
     Return_if_fail(write_file(in, out));
 
     If_fail(Err_fclose(out))
-	throw prcserror << "Failed writing to file "
+	pthrow prcserror << "Failed writing to file "
 			 << squote(filename) << perror;
 
     return NoError;
@@ -818,7 +836,7 @@ PrVoidError fs_move_filename(const char* infile, const char* outfile)
 	return NoError;
 
     if (errno != EXDEV)
-	throw prcserror << "Rename failed on " << squote(infile) << " to "
+	pthrow prcserror << "Rename failed on " << squote(infile) << " to "
 			<< squote(outfile) << perror;
 
     return fs_copy_filename(infile, outfile);
@@ -829,11 +847,11 @@ PrVoidError fs_copy_filename(const char* infile, const char* outfile)
     FILE *in, *out;
 
     If_fail(in << Err_fopen(infile, "r"))
-	throw prcserror << "Failed opening file " << squote(infile)
+	pthrow prcserror << "Failed opening file " << squote(infile)
 	                << " for reading" << perror;
 
     If_fail(out << Err_fopen(outfile, "w"))
-	throw prcserror << "Failed opening file " << squote(outfile)
+	pthrow prcserror << "Failed opening file " << squote(outfile)
 		        << " for writing" << perror;
 
     Return_if_fail(write_file(in, out));
@@ -841,7 +859,7 @@ PrVoidError fs_copy_filename(const char* infile, const char* outfile)
     fclose(in);
 
     If_fail(Err_fclose(out))
-	throw prcserror << "Failed writing file " << squote(outfile)
+	pthrow prcserror << "Failed writing file " << squote(outfile)
 			 << perror;
 
     return NoError;
@@ -879,25 +897,31 @@ PrVoidError write_file(FILE* in, FILE* out)
 
     opt_buffer_1(-1);
 
-    while((c = fread(Buffer1, 1, Buffer1Len, in)) > 0) {
-        fwrite(Buffer1, 1, c, out);
+    for (;;) {
+        If_fail (c << Err_fread(Buffer1, Buffer1Len, in))
+	  pthrow prcserror << "Read failure" << perror;
+
+	if (c == 0)
+	  break;
+
+        If_fail (Err_fwrite(Buffer1, c, out))
+	    pthrow prcserror << "Write failure" << perror;
     }
 
-    if(ferror(in))
-        throw prcserror << "Read failure" << perror;
-
-    if(ferror(out))
-        throw prcserror << "Write failure" << perror;
-
     return NoError;
 }
 
 bool empty_file(FILE* empty)
 {
+    int c;
+
     opt_buffer_1(-1);
 
-    while(fread(Buffer1, 1, Buffer1Len, empty) > 0)
-        /* do nothing */;
+    do {
+      If_fail (c << Err_fread(Buffer1, Buffer1Len, empty))
+	c = -1;
+    } while (c > 0);
+
     return true;
 }
 
@@ -905,7 +929,7 @@ PrVoidError fs_zero_file(const char* name)
 {
     /* use open so that if FILE doesn't exist create it. */
     If_fail(Err_open(name, O_TRUNC|O_CREAT, 0644))
-        throw prcserror << "Failed truncating file "<< squote(name) << perror;
+        pthrow prcserror << "Failed truncating file "<< squote(name) << perror;
 
     return NoError;
 }
@@ -1016,6 +1040,43 @@ NprVoidError Err_rename(const char* a, const char* b)
 	return NoError;
 }
 
+NprIntError Err_fread(void* buf, size_t nbytes, FILE* f)
+{
+  int c;
+
+  do {
+    errno = 0;
+    clearerr(f);
+    c = fread (buf, 1, nbytes, f);
+  } while (c <= 0 && ferror(f) && errno == EINTR);
+
+  if (c <= 0 && ferror (f))
+    return ReadFailure;
+  else
+    return c;
+}
+
+NprVoidError Err_fwrite(const void* buf, size_t nbytes, FILE* out)
+{
+  int c;
+
+  while (nbytes > 0) {
+    do {
+      errno = 0;
+      clearerr(out);
+      c = fwrite (buf, 1, nbytes, out);
+    } while (c <= 0 && ferror(out) && errno == EINTR);
+
+    if (c <= 0 && ferror(out))
+      return WriteFailure;
+
+    nbytes -= c;
+    buf = ((const char*)buf) + c;
+  }
+
+  return NoError;
+}
+
 NprIntError Err_read(int fd, void* buf, size_t nbytes)
 {
     int c;
@@ -1222,7 +1283,7 @@ PrConstCharPtrError find_real_filename(const char* filename)
     struct stat sbuf;
 
     if (lstat(filename, &sbuf) < 0)
-	throw prcserror << "Lstat failed on "
+	pthrow prcserror << "Lstat failed on "
 			<< squote(filename)
 			<< perror;
 
@@ -1268,11 +1329,18 @@ PrVoidError WriteableFile::copy(FILE* copy_me)
 
     int c;
 
-    while((c = fread(Buffer1, 1, Buffer1Len, copy_me)) > 0)
-	os.write(Buffer1, c);
+    for (;;) {
+        If_fail (c << Err_fread(Buffer1, Buffer1Len, copy_me))
+	  pthrow prcserror << "Read failure" << perror;
 
-    if (os.bad())
-	throw prcserror << "Error writing " << squote(temp_name) << perror;
+	if (c == 0)
+	  break;
+
+	os.write (Buffer1, c);
+
+	if (os.bad())
+	    pthrow prcserror << "Error writing " << squote(temp_name) << perror;
+    }
 
     return NoError;
 }
@@ -1282,7 +1350,7 @@ PrVoidError WriteableFile::copy(const char* copy_me)
     FILE* it;
 
     If_fail(it << Err_fopen(copy_me, "r"))
-	throw prcserror << "Open failed on " << squote(copy_me) << perror;
+	pthrow prcserror << "Open failed on " << squote(copy_me) << perror;
 
     PrVoidError ret = copy(it);
 
@@ -1341,6 +1409,9 @@ const char* get_environ_var (const char* var)
 	    if (!env)
 		env = env_names[i].defval;
 
+	    if (env && !env[0])
+	      env = NULL;
+
 	    env_table.insert (env_names[i].var, env);
 	}
     }
@@ -1352,7 +1423,7 @@ const char* get_environ_var (const char* var)
 
     env = getenv (var);
 
-    if (env) {
+    if (env && *env) {
 	env_table.insert (var, env);
 
 	return env;
