@@ -16,11 +16,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: prcs.cc 1.23.1.1.1.14.1.18.1.8.1.16.1.1.1.7.1.24 Wed, 22 Oct 1997 15:29:15 -0700 jmacd $
+ * $Id: prcs.cc 1.23.1.1.1.14.1.18.1.8.1.16.1.1.1.7.1.32 Wed, 14 Oct 1998 20:50:34 -0700 jmacd $
  */
 
 /* $Format: "static const char prcs_version_id[] = \"$ProjectVersion$ $ProjectAuthor$ $ProjectDate$\";"$ */
-static const char prcs_version_id[] = "1.2-release.17 jmacd Wed, 14 Jan 1998 23:24:44 -0800";
+static const char prcs_version_id[] = "1.2-release.80 jmacd Mon, 02 Nov 1998 23:28:03 -0800";
 
 #include "prcs.h"
 #include "lock.h"
@@ -33,6 +33,10 @@ static const char prcs_version_id[] = "1.2-release.17 jmacd Wed, 14 Jan 1998 23:
 #include "repository.h"
 #include "system.h"
 
+#if defined(__BEOS__)
+#include <KernelKit.h>
+#endif
+
 extern "C" {
 #include <signal.h>
 #include <sys/time.h>
@@ -40,7 +44,7 @@ extern "C" {
 }
 
 /* $Format: "const char prcs_version_string[] = \"$ReleaseVersion$\";"$ */
-const char prcs_version_string[] = "1.2.2";
+const char prcs_version_string[] = "1.2.11";
 
 /* The following classes are only used inside this file, and oraganize
  * several arrays of information used to select the command and
@@ -151,6 +155,7 @@ static CommandOptionPair const command_options[] = {
     { 'd', "--delete", &option_populate_delete },
     { 'z', "--compress", &option_package_compress },
     { 'p', "--preserve", &option_preserve_permissions },
+    { 'P', "--exclude-project-file", &option_exclude_project_file },
     { 'u', "--unlink", &option_unlink },
     { 's', "--skilled-merge", &option_skilled_merge },
     { SORT_CHAR, "--sort", &option_sort },
@@ -189,6 +194,7 @@ static struct option const long_options[] = {
   {"preserve-permissions",    no_argument, 0, 'p'},
   {"preserve",                no_argument, 0, 'p'},
   {"permissions",             no_argument, 0, 'p'},
+  {"exclude-project-file",    no_argument, 0, 'P'},
   {"revision",                required_argument, 0, 'r'},
   {"unlink",                  no_argument, 0, 'u'},
   {"skilled-merge",           no_argument, 0, 's'},
@@ -200,7 +206,7 @@ static struct option const long_options[] = {
   {"pipe",                    no_argument, 0, PIPE_CHAR},
   {0,0,0,0}};
 
-static const char prcs_optstring[] = "+vhHLlnqfR:r:NkidzFpruj:s"
+static const char prcs_optstring[] = "+vhHLlnqfR:r:NkidzFpPruj:s"
                                      EXECUTE_STR PLAINFORMAT_STR SORT_STR ":"
 #ifdef PRCS_DEVEL
 "Dt:"
@@ -215,11 +221,11 @@ static PrcsCommand commands[] = {
 
     {"checkout", "@", "@", "@", NoFlags,
      checkout_command, checkout_help_string,
-     OneRevisionArg, InsureProjectName, "up", NoDiffArgs, -1 },
+     OneRevisionArg, InsureProjectName, "upP", NoDiffArgs, -1 },
 
     {"diff", "", "", "@", NoFlags,
      diff_command, diff_help_string,
-     TwoRevisionArgs, InsureProjectName, "NkF", ReallyDiffArgs, -1 },
+     TwoRevisionArgs, InsureProjectName, "NkFP", ReallyDiffArgs, -1 },
 
     {"merge", "", "@", "@", NoFlags,
      merge_command, merge_help_string,
@@ -322,6 +328,10 @@ static const int unfriendly_signals[] = {
     SIGBUS, SIGFPE, SIGILL, SIGQUIT, SIGSEGV, 0
 };
 
+#if defined(__BEOS__)
+static thread_id beos_alarm_id = 0;
+#endif
+
 /* End static data */
 
 static PrVoidError check_system_command(CommandNamePair com,
@@ -339,17 +349,17 @@ static PrVoidError check_system_command(CommandNamePair com,
     Return_if_fail(com.command->open(true, false));
 
     If_fail(read_string(com.command->standard_out(), &output))
-	throw prcserror << "Read failed from " << com.name << perror;
+	pthrow prcserror << "Read failed from " << com.name << perror;
 
     Return_if_fail_if_ne(com.command->close(), 0) {
-	throw prcserror << com.name << ' ' << arg
+	pthrow prcserror << com.name << ' ' << arg
 			<< " failed, " << com.name
 			<< " is probably too old for PRCS to use"
 			<< dotendl;
     }
 
     if(strncmp(output.cast(), expected, strlen (expected)) != 0) {
-	throw prcserror << com.name
+	pthrow prcserror << com.name
 			 << " installation is too old, please install "
 			 << expected << dotendl;
     }
@@ -469,7 +479,7 @@ static PrVoidError check_major_version_name(const char* name)
 {
     if (name[0] && strcmp(name, "@") != 0 &&
 	VC_check_token_match(name, "label") <= 0)
-	throw prcserror << "Illegal major version name "
+	pthrow prcserror << "Illegal major version name "
 			 << squote(name)
 			 << dotendl;
 
@@ -480,7 +490,7 @@ static PrVoidError check_minor_version_name(const char* name)
 {
     if (name[0] && strcmp(name, "@") != 0 &&
 	VC_check_token_match(name, "number") <= 0)
-	throw prcserror << "Illegal minor version name "
+	pthrow prcserror << "Illegal minor version name "
 			 << squote(name)
 			 << dotendl;
 
@@ -492,7 +502,7 @@ static PrPrcsExitStatusError invoke_command(PrcsCommand *c)
     bool warning = false;
 
     if (c->n_files_allowed >= 0 && cmd_filenames_count != c->n_files_allowed) {
-	throw prcserror << "This command requires " << c->n_files_allowed
+	pthrow prcserror << "This command requires " << c->n_files_allowed
 			<< " file-or-dir arguments" << dotendl;
     }
 
@@ -511,7 +521,7 @@ static PrPrcsExitStatusError invoke_command(PrcsCommand *c)
 	if (c->revision_flags & NoDefault &&
 	    !cmd_version_specifier_major &&
 	    !cmd_version_specifier_minor)
-	    throw prcserror << "You must specify a revision argument with "
+	    pthrow prcserror << "You must specify a revision argument with "
 			    << squote ("-r") << " for this command" << dotendl;
 
 	if (!cmd_version_specifier_major && !cmd_version_specifier_minor) {
@@ -523,7 +533,7 @@ static PrPrcsExitStatusError invoke_command(PrcsCommand *c)
 	    cmd_version_specifier_minor = c->defmin_maj_only;
 
 	if (!cmd_version_specifier_minor[0] && cmd_version_specifier_major[0])
-	    throw prcserror << "It is illegal to specify a null minor "
+	    pthrow prcserror << "It is illegal to specify a null minor "
 		"version without a null major version" << dotendl;
 
 	if (!(c->revision_flags & AllowWildCards)) {
@@ -597,7 +607,7 @@ static PrVoidError set_project_name(const char* specifier0)
 	Return_if_fail(prj << guess_prj_name(specifier0));
 
 	if (prj == NULL) {
-	    throw prcserror << "Cannot determine project name, there must be a single "
+	    pthrow prcserror << "Cannot determine project name, there must be a single "
 		"project file in the directory " << squote(specifier0) << dotendl;
 	}
 
@@ -608,7 +618,7 @@ static PrVoidError set_project_name(const char* specifier0)
 	delete prj;
 
     } else if (specifier0[len - 1] == '/') {
-	throw prcserror << "Directory " << squote(specifier0)
+	pthrow prcserror << "Directory " << squote(specifier0)
 			<< " does not exist, cannot determine project name" << dotendl;
     } else if (last_slash) {
 	project_path->assign(specifier0);
@@ -662,7 +672,7 @@ static PrIntError determine_project(PrcsCommand* command, int argc, char** argv)
 	    Return_if_fail(set_project_name("./"));
 
 	} else if (command->project_arg_type != OptionalProjectName) {
-	    throw prcserror << "Missing operand" << dotendl;
+	    pthrow prcserror << "Missing operand" << dotendl;
 	}
     }
 
@@ -696,6 +706,9 @@ static const char* correct_cmd_filename(char* cmd_filename)
 	    correct.truncate(cmd_root_project_path_len);
     }
 
+    while (correct.length() > 0 && correct.last_index() == '/')
+      correct.truncate (correct.length() - 1);
+
     return p_strdup(correct);
 
 nowayman:
@@ -816,6 +829,7 @@ static bool read_command_line(int argc, char** argv)
 	case 'L': option_really_long_format = 1; break;
 	case 'n': option_report_actions = 1; break;
 	case 'p': option_preserve_permissions = 1; break;
+	case 'P': option_exclude_project_file = 1; break;
 	case 'u': option_unlink = 1; break;
 	case 'R': cmd_repository_path = optarg; break;
 	case 'j':
@@ -944,6 +958,7 @@ static bool read_command_line(int argc, char** argv)
 
     i = 0;
     cmd_diff_options_given = 0;
+    cmd_diff_options_count = -1;
 
     while(i < cmd_filenames_count) {
 	if(strcmp(cmd_filenames_given[i], "--") == 0) {
@@ -954,13 +969,13 @@ static bool read_command_line(int argc, char** argv)
 	i += 1;
     }
 
-    if (command->diff_arg_type == NoDiffArgs && cmd_diff_options_count > 0) {
+    if (command->diff_arg_type == NoDiffArgs && cmd_diff_options_count >= 0) {
 	prcserror << "This command does not accept arguments following "
 		  << squote ("--") << dotendl;
 	exit(2);
     }
 
-    if (cmd_diff_options_count == 0 && command->diff_arg_type == ReallyDiffArgs) {
+    if (cmd_diff_options_count < 0 && command->diff_arg_type == ReallyDiffArgs) {
 	const char* pdo = get_environ_var ("PRCS_DIFF_OPTIONS");
 
 	if (pdo) {
@@ -1051,6 +1066,12 @@ static PrVoidError clean_up(bool signaled)
 	temp_directory = NULL;
     }
 
+#if defined(__BEOS__)
+    if (beos_alarm_id > 0) {
+        kill_thread(beos_alarm_id);
+        beos_alarm_id = 0;
+    }
+#endif
     return NoError;
 }
 
@@ -1074,12 +1095,29 @@ static void clean_up_handler(SIGNAL_ARG_TYPE)
 
 static void alarm_handler(SIGNAL_ARG_TYPE)
 {
+
     CleanupHandler* handler = alarm_handler_list;
 
     for (; handler; handler = handler->next_handler)
 	handler->handler(handler->data, false);
 }
 
+#if defined(__BEOS__)
+static int32 beos_alarm_thread (void * data)
+{
+    while (1) {
+        snooze( (STALE_LOCK_TIMEOUT - 30) * 1000000 );
+        alarm_handler(SIGALRM);
+    }
+}
+#endif
+
+static void chld_handler(SIGNAL_ARG_TYPE)
+{
+  /* Silly Solaris does not interrupt select() for SIGCHLD when there
+   * is no handler!!! */
+}
+
 static void handle_signals()
 {
     struct sigaction act;
@@ -1108,7 +1146,17 @@ static void handle_signals()
     act.sa_mask = signal_mask;
     sigaction(SIGCONT, &act, NULL);
 
+    /* SIGCHLD */
+    act.sa_handler = chld_handler;
+    act.sa_mask = signal_mask;
+    sigaction (SIGCHLD, &act, NULL);
+
+#if !defined(__BEOS__)
     /* SIGALRM */
+    act.sa_handler = alarm_handler;
+    act.sa_mask = signal_mask;
+    sigaction (SIGALRM, &act, NULL);
+
     struct itimerval itval;
 
     itval.it_interval.tv_sec = STALE_LOCK_TIMEOUT - 30;
@@ -1116,10 +1164,9 @@ static void handle_signals()
     itval.it_value.tv_sec = STALE_LOCK_TIMEOUT - 30;
     itval.it_value.tv_usec = 0;
 
-    act.sa_handler = alarm_handler;
-    act.sa_mask = signal_mask;
-    sigaction (SIGALRM, &act, NULL);
     setitimer (ITIMER_REAL, &itval, NULL);
+#endif
+
 }
 
 /* Temp dirs */
@@ -1135,11 +1182,11 @@ static PrVoidError make_temp_dir()
 	tmpdir = "/tmp";
 
     if(!fs_is_directory(tmpdir) || !fs_file_wrandex(tmpdir))
-	throw prcserror << "Can't access temp directory "
+	pthrow prcserror << "Can't access temp directory "
 		        << squote(tmpdir) << perror;
 
     if(tmpdir[0] != '/')
-	throw prcserror << "TMPDIR must be an absolute path" << dotendl;
+	pthrow prcserror << "TMPDIR must be an absolute path" << dotendl;
 
     dir = new Dstring(tmpdir);
     if (dir->index(dir->length() - 1) != '/')
@@ -1158,7 +1205,7 @@ static PrVoidError make_temp_dir()
     temp_directory = *dir;
 
     If_fail(Err_mkdir(temp_directory, (mode_t)0700))
-	throw prcserror << "Mkdir failed for file " << squote(temp_directory) << perror;
+	pthrow prcserror << "Mkdir failed for file " << squote(temp_directory) << perror;
 
     return NoError;
 }
@@ -1195,6 +1242,27 @@ int main(int argc, char** argv)
 
     handle_signals();
 
+#if defined(__BEOS__)
+    /* We simulate the alarm() with a dedicated thread - this makes
+     * executing the alarm signal handler much safer.
+     */
+    beos_alarm_id = spawn_thread( beos_alarm_thread, "prcs alarm thread"
+                                , B_NORMAL_PRIORITY, NULL);
+    if (beos_alarm_id < 0) {
+        prcserror << "Can't create the alarm thread: "
+                  << strerror(beos_alarm_id)
+                  << dotendl;
+        exit(2);
+    }
+    status_t beos_thread_state = resume_thread(beos_alarm_id);
+    if (B_NO_ERROR != beos_thread_state) {
+        prcserror << "Can't start the alarm thread: "
+                  << strerror(beos_thread_state)
+                  << dotendl;
+        exit(2);
+    }
+#endif
+
     if(VC_check_token_match(get_login(), "login") <= 0) {
 	prcserror << "Your login name, " << squote(get_login()) << ", may confuse the lexer" << dotendl;
 	bug();
@@ -1214,7 +1282,7 @@ int main(int argc, char** argv)
 	exit(2);
 
     if(!option_force_resolution && !isatty(STDIN_FILENO) ) {
-	prcserror << "Please specify -f to run PRCS without a controlling terminal"
+        prcserror << "Please specify -f to run PRCS without a controlling terminal"
 		  << dotendl;
 	exit(2);
     }
