@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: projdesc.cc 1.9.1.1.1.4.1.17.1.5.1.17.3.9.1.20 Sun, 12 Oct 1997 17:11:49 -0700 jmacd $
+ * $Id: projdesc.cc 1.9.1.1.1.4.1.17.1.5.1.17.3.9.1.31 Wed, 14 Oct 1998 20:50:34 -0700 jmacd $
  */
 
 extern "C" {
@@ -36,6 +36,7 @@ extern "C" {
 #include "checkout.h"
 #include "memseg.h"
 #include "setkeys.h"
+#include "populate.h"
 
 #define prj_lex_val prjtext
 #define prj_lex_len prjleng
@@ -90,7 +91,7 @@ const char *ProjectDescriptor::project_file_path() const { return _project_file_
 const char *ProjectDescriptor::project_aux_path()  const { return _project_aux_path; }
 const char *ProjectDescriptor::project_path()      const { return _project_path; }
 const char *ProjectDescriptor::full_version()      const { return _full_version; }
-DstringPtrArray* ProjectDescriptor::populate_ignore() const { return _populate_ignore; };
+OrderedStringTable* ProjectDescriptor::populate_ignore() const { return _populate_ignore; };
 ProjectVersionData* ProjectDescriptor::project_version_data() const { return _project_version_data; }
 FileEntryPtrArray* ProjectDescriptor::file_entries() { return _file_entries; }
 PrettyOstream& ProjectDescriptor::merge_log() const { return *_log; }
@@ -108,7 +109,11 @@ ProjectDescriptor::~ProjectDescriptor()
     }
 
     if (_log_pstream) {
+#ifdef __GNUG__
 	if (_log_pstream->sync())
+#else
+        if (_log_pstream->pubsync())
+#endif
 	    prcserror << "warning: Write to merge log failed" << perror;
 
 	delete _log_pstream;
@@ -132,6 +137,7 @@ ProjectDescriptor::~ProjectDescriptor()
     if (_populate_ignore)  delete _populate_ignore;
     if (_quick_elim)       delete _quick_elim;
     if (_project_keywords) delete _project_keywords;
+    if (_project_keywords_extra) delete _project_keywords_extra;
     if (_keyword_id)       delete _keyword_id;
     if (_keyword_pheader)  delete _keyword_pheader;
     if (_keyword_pversion) delete _keyword_pversion;
@@ -152,9 +158,13 @@ ProjectDescriptor::ProjectDescriptor()
      _files_insertion_point(NULL),
      _project_version_point(NULL),
      _end_of_buffer_point(NULL),
+     _project_keywords_point(NULL),
+     _populate_ignore_point(NULL),
      _merge_parents_marker(0, 0),
      _new_merge_parents_marker(0, 0),
      _descends_from_marker(0, 0),
+     _project_keywords_marker(0, 0),
+     _populate_ignore_marker(0, 0),
      _deleted_markers(NULL),
      _repository_entry(NULL),
      _file_entries(NULL),
@@ -179,6 +189,7 @@ ProjectDescriptor::ProjectDescriptor()
      _merge_parents(NULL),
      _new_merge_parents(NULL),
      _project_keywords(NULL),
+     _project_keywords_extra(NULL),
      _keyword_id(NULL),
      _keyword_pheader(NULL),
      _keyword_pversion(NULL),
@@ -186,6 +197,7 @@ ProjectDescriptor::ProjectDescriptor()
      _merge_selected_major(NULL),
      _merge_selected_minor(NULL),
      _append_merge_parents(false),
+     _alter_popkey(false),
      _merge_complete(false),
      _merge_incomplete(false),
      _file_name_table(NULL),
@@ -194,7 +206,7 @@ ProjectDescriptor::ProjectDescriptor()
 
 PrVoidError ProjectDescriptor::bad_project_file(const Sexp *s, const char* mes)
 {
-    throw prcserror << fileline(_prj_source_name, s->line_number())
+    pthrow prcserror << fileline(_prj_source_name, s->line_number())
 		    << "Invalid project file, " << mes << dotendl;
 }
 
@@ -260,7 +272,7 @@ PrVoidError ProjectDescriptor::init_from_file(const char* full_name,
 
     if(infile0 == NULL) {
         if((infile = fopen(filename, "r")) == NULL)
-	    throw prcserror << "Can't open file " << squote(filename) << perror;
+	    pthrow prcserror << "Can't open file " << squote(filename) << perror;
     } else {
 	infile = infile0;
     }
@@ -272,6 +284,9 @@ PrVoidError ProjectDescriptor::init_from_file(const char* full_name,
 	_merge_parents = new MergeParentEntryPtrArray;
     }
 
+    _populate_ignore = new OrderedStringTable;
+    _project_keywords = new KeywordTable;
+    _project_keywords_extra = new OrderedStringTable;
     _file_entries = new FileEntryPtrArray;
     _deleted_markers = new ConstListMarkerPtrArray;
     _attrs_table = new AttrsTable (attrs_hash, attrs_equal);
@@ -327,8 +342,13 @@ PrVoidError ProjectDescriptor::init_merge_log()
 
     _log_stream = new filebuf;
 
-    if (!_log_stream->open(logname, "a"))
-	throw prcserror << "Failed opening merge log file " << squote(logname) << perror;
+#ifndef __GNUG__
+#  define APPENDTAG ios::out|ios::app
+#else
+#  define APPENDTAG "a"
+#endif
+    if (!_log_stream->open(logname, APPENDTAG))
+	pthrow prcserror << "Failed opening merge log file " << squote(logname) << perror;
 
     _log_pstream = new PrettyStreambuf (_log_stream, &option_report_actions);
 
@@ -528,6 +548,9 @@ PrVoidError ProjectDescriptor::descends_from_prj_entry_func(const Sexp* s)
 
 PrVoidError ProjectDescriptor::populate_ignore_prj_entry_func(const Sexp* s)
 {
+    _populate_ignore_marker = ListMarker (s->start_index(), s->end_index());
+    _populate_ignore_point = new Estring (this, s->end_index());
+
     if(s->length() != 2 || !s->cadr()->is_pair())
 	return bad_project_file(s, "malformed Populate-Ignore entry");
 
@@ -538,11 +561,10 @@ PrVoidError ProjectDescriptor::populate_ignore_prj_entry_func(const Sexp* s)
 	    return bad_project_file(t, "expected a string");
     }
 
-    if (_read_flags & KeepPopulateIgnoreData && s->cadr()->length() > 0) {
-	_populate_ignore = new DstringPtrArray;
-
-	foreach_sexp (t_ptr, s->cadr())
-	    _populate_ignore->append ((*t_ptr)->key());
+    foreach_sexp (t_ptr, s->cadr()) {
+	Dstring* ds = (*t_ptr)->key();
+	const char* t = ds->cast();
+	_populate_ignore->insert (t, t);
     }
 
     return NoError;
@@ -550,16 +572,8 @@ PrVoidError ProjectDescriptor::populate_ignore_prj_entry_func(const Sexp* s)
 
 PrVoidError ProjectDescriptor::project_keywords_prj_entry_func (const Sexp* s)
 {
-    KeywordTable *tmp = NULL;
-    KeywordTable *table;
-
-    if (_read_flags & KeepKeywords) {
-	_project_keywords = new KeywordTable;
-	table = _project_keywords;
-    } else {
-	tmp = new KeywordTable;
-	table = tmp;
-    }
+    _project_keywords_marker = ListMarker (s->start_index(), s->end_index());
+    _project_keywords_point = new Estring (this, s->end_index());
 
     foreach_sexp (t_ptr, s->cdr()) {
 	const Sexp *t = *t_ptr;
@@ -569,22 +583,21 @@ PrVoidError ProjectDescriptor::project_keywords_prj_entry_func (const Sexp* s)
 	   t->car()->is_pair() ||
 	   t->cadr()->is_pair() ||
 	   t->car()->key()->type() != EsNameLiteral)
-	    return bad_project_file(t, "malformed Project-Version entry");
+	    return bad_project_file(t, "malformed Project-Keywords entry");
 
 	if (is_builtin_keyword (t->car()->key()->cast(), t->car()->key()->length()))
 	    return bad_project_file(t, "Project-Keywords may not contain a builtin keyword");
 
-	if (table->isdefined (t->car()->key()->cast()))
+	if (_project_keywords->isdefined (t->car()->key()->cast()))
 	    return bad_project_file(t, "duplicate keyword");
 
-	if (strchr (t->car()->key()->cast(), '\n'))
-	    return bad_project_file(t, "keyword value may not contain a newline");
+	if (strchr (t->cadr()->key()->cast(), '\n'))
+	    return bad_project_file(t, "keyword value may not contain a `$' character");
 
-	table->insert (t->car()->key()->cast(), t->cadr()->key()->cast());
+	_project_keywords->insert (t->car()->key()->cast(), t->cadr()->key()->cast());
+	_project_keywords_extra->insert (t->car()->key()->cast(), t->cadr()->key()->cast());
     }
 
-    if (tmp) delete tmp;
-
     return NoError;
 }
 
@@ -971,13 +984,13 @@ PrVoidError ProjectDescriptor::verify_project_file()
 
 	if(filenames.isdefined(fe_name)) {
 	    if(strcmp(fe_name, project_file_name) == 0)
-		throw prcserror << "Entry " << squote(fe_name)
+		pthrow prcserror << "Entry " << squote(fe_name)
 				<< " duplicates project file name" << dotendl;
 	    else if (strcmp(fe_name, project_aux_name) == 0)
-		throw prcserror << "Entry " << squote(fe_name)
+		pthrow prcserror << "Entry " << squote(fe_name)
 				<< " duplicates project auxiliary file name" << dotendl;
 	    else
-		throw prcserror << "Entry " << squote(fe_name)
+		pthrow prcserror << "Entry " << squote(fe_name)
 				<< " is a duplicate file name" << dotendl;
 	}
 
@@ -990,10 +1003,10 @@ PrVoidError ProjectDescriptor::verify_project_file()
 
 	if(descriptors.isdefined(fe_desc)) {
 	    if(strcmp(fe_desc, project_descriptor) == 0)
-		throw prcserror << "Entry " << squote(fe_name)
+		pthrow prcserror << "Entry " << squote(fe_name)
 				<< " duplicates project file descriptor" << dotendl;
 	    else
-		throw prcserror << "Entry " << squote(fe_name)
+		pthrow prcserror << "Entry " << squote(fe_name)
 				<< " contains a duplicate file descriptor" << dotendl;
 	}
 
@@ -1012,7 +1025,7 @@ PrVoidError ProjectDescriptor::verify_project_file()
 		FileType* ft = filenames.lookup(name);
 
 		if (ft && (*ft) != Directory)
-		    throw prcserror << "Non-directory filename " << squote(name)
+		    pthrow prcserror << "Non-directory filename " << squote(name)
 				    << " is prefix to another file" << dotendl;
 	    }
 	}
@@ -1149,18 +1162,18 @@ PrProjectVersionDataPtrError ProjectDescriptor::check_version_name(const char* n
     const char* this_version_minor = minor_version_of(name);
 
     if(!this_version_major)
-	throw prcserror << fileline(_prj_source_name, line)
+	pthrow prcserror << fileline(_prj_source_name, line)
 			<< "Illegal version number " << name << dotendl;
 
     if(VC_check_token_match(this_version_major, "label") <= 0 ||
        VC_check_token_match(this_version_minor, "number") <= 0)
-	throw prcserror << fileline(_prj_source_name, line)
+	pthrow prcserror << fileline(_prj_source_name, line)
 			<< "Illegal version number " << name << dotendl;
 
     data = repository_entry()->lookup_version(this_version_major, this_version_minor);
 
     if (!data)
-	throw prcserror << fileline(_prj_source_name, line)
+	pthrow prcserror << fileline(_prj_source_name, line)
 			<< "Version " << name
 			<< " not found in repository" << dotendl;
 
@@ -1188,11 +1201,61 @@ PrVoidError ProjectDescriptor::adjust_merge_parents()
 /* Keywords
  */
 
+void ProjectDescriptor::add_keyword (const char* key, const char* val)
+{
+    _alter_popkey = true;
+
+    ASSERT (! _project_keywords_extra->lookup (key), "blah");
+
+    _project_keywords->insert (key, val);
+    _project_keywords_extra->insert (key, val);
+}
+
+void ProjectDescriptor::rem_keyword (const char* key)
+{
+    _alter_popkey = true;
+
+    ASSERT (_project_keywords_extra->lookup (key), "blah");
+
+    _project_keywords->remove (key);
+    _project_keywords_extra->remove (key);
+}
+
+void ProjectDescriptor::set_keyword (const char* key, const char* val)
+{
+    _alter_popkey = true;
+
+    ASSERT (_project_keywords_extra->lookup (key), "blah");
+
+    _project_keywords->insert (key, val);
+    _project_keywords_extra->insert (key, val);
+}
+
+void ProjectDescriptor::add_populate_ignore (const char* key)
+{
+    _alter_popkey = true;
+
+    ASSERT (! _populate_ignore->lookup (key), "blah");
+
+    _populate_ignore->insert (key, key);
+}
+
+void ProjectDescriptor::rem_populate_ignore (const char* key)
+{
+    _alter_popkey = true;
+
+    ASSERT (_populate_ignore->lookup (key), "blah");
+
+    _populate_ignore->remove (key);
+}
+
+OrderedStringTable* ProjectDescriptor::project_keywords_extra() const
+{
+    return _project_keywords_extra;
+}
+
 KeywordTable* ProjectDescriptor::project_keywords(FileEntry* fe, bool setting)
 {
-    if (!_project_keywords)
-	_project_keywords = new KeywordTable;
-
     if (!_keyword_id) {
 	_keyword_id = new Dstring;
 	_keyword_pversion = new Dstring;
@@ -1289,6 +1352,17 @@ void ProjectDescriptor::append_directory (const char* working_path)
     _files_insertion_point->append (" () :directory)");
 }
 
+void ProjectDescriptor::append_file_deletion (FileEntry* fe)
+{
+    _files_insertion_point->append ("\n  ; (");
+    _files_insertion_point->append_protected (fe->working_path());
+    _files_insertion_point->append (" ()");
+
+    fe->file_attrs()->print_to_string (_files_insertion_point, true);
+
+    _files_insertion_point->append (")");
+}
+
 void ProjectDescriptor::append_file (FileEntry* fe)
 {
     _files_insertion_point->append ("\n  (");
@@ -1369,7 +1443,7 @@ void PrcsAttrs::print (ostream& os, bool lead_if) const
     }
 }
 
-bool PrcsAttrs::regex_matches (regex_t *re) const
+bool PrcsAttrs::regex_matches (reg2ex2_t *re) const
 {
     foreach (ds_ptr, &_vals, DstringPtrArray::ArrayIterator) {
 	if (prcs_regex_matches ((*ds_ptr)->cast(), re))
@@ -1397,14 +1471,14 @@ PrVoidError ProjectDescriptor::validate_attrs (const PrcsAttrs *attrs, const cha
 	const AttrDesc* ad = is_file_attribute (an.cast(), an.length());
 
 	if (!ad)
-	    throw prcserror  << "Unrecognized attribute " << an << " for file " << squote(name) << dotendl;
+	    pthrow prcserror  << "Unrecognized attribute " << an << " for file " << squote(name) << dotendl;
 
 	switch (ad->type) {
 	case RealFileAttr:
 	case DirectoryAttr:
 	case SymlinkAttr:
 	    if (seen_type)
-		throw prcserror << "Duplicate file type attribute for file " << squote(name) << dotendl;
+		pthrow prcserror << "Duplicate file type attribute for file " << squote(name) << dotendl;
 	    seen_type = true;
 	    break;
 	case Mergerule1Attr:
@@ -1422,12 +1496,12 @@ PrVoidError ProjectDescriptor::validate_attrs (const PrcsAttrs *attrs, const cha
 	case Mergerule13Attr:
 	case Mergerule14Attr:
 	    if (av.length() != 1 || strchr("andrm", av.index(0)) == NULL)
-		throw prcserror << "Illegal merge action " << av << dotendl;
+		pthrow prcserror << "Illegal merge action " << av << dotendl;
 	    break;
 	case MergetoolAttr:
 	case DifftoolAttr:
 	    if (av.length() < 1)
-		throw prcserror << "Illegal tool name " << av << dotendl;
+		pthrow prcserror << "Illegal tool name " << av << dotendl;
 	default:
 	    break;
 	}
@@ -1536,7 +1610,7 @@ bool attrs_equal(const PrcsAttrs*const& x, const PrcsAttrs*const& y)
     return true;
 }
 
-extern int hash(const char*& s, int M);
+extern int hash(const char *const& s, int M);
 
 int attrs_hash(const PrcsAttrs*const & s, int M)
 {
@@ -1576,6 +1650,57 @@ PrVoidError ProjectDescriptor::write_project_file(const char* filename)
     created_by_minor()->assign_int(prcs_version_number[1]);
     created_by_micro()->assign_int(prcs_version_number[2]);
 
+    if (_alter_popkey) {
+	_deleted_markers->append (& _populate_ignore_marker);
+	_deleted_markers->append (& _project_keywords_marker);
+
+	int first = 0;
+
+	_project_keywords_point->append ("(Project-Keywords");
+	_populate_ignore_point->append  ("(Populate-Ignore (");
+
+	int ks = _project_keywords_extra->key_array()->length();
+
+	for (int i = 0; i < ks; i += 1) {
+	    int nspaces = (first * strlen ("(Project-Keywords")) + 1;
+
+	    for (int j = 0; j < nspaces; j += 1)
+		_project_keywords_point->append (" ");
+
+	    _project_keywords_point->append ("(");
+	    _project_keywords_point->append_protected (_project_keywords_extra->key_array()->index(i));
+	    _project_keywords_point->append (" ");
+	    _project_keywords_point->append_string (_project_keywords_extra->data_array()->index(i));
+	    _project_keywords_point->append (")");
+
+	    if (i < (ks - 1))
+		_project_keywords_point->append ("\n");
+
+	    first = 1;
+	}
+
+	first = 0;
+
+	ks = _populate_ignore->key_array()->length();
+
+	for (int i = 0; i < ks; i += 1) {
+	    int nspaces = (first * strlen ("(PopulateIgnore ( "));
+
+	    for (int j = 0; j < nspaces; j += 1)
+		_populate_ignore_point->append (" ");
+
+	    _populate_ignore_point->append_string (_populate_ignore->key_array()->index(i));
+
+	    if (i < (ks - 1))
+		_populate_ignore_point->append ("\n");
+
+	    first = 1;
+	}
+
+	_project_keywords_point->append (")");
+	_populate_ignore_point->append ("))");
+    }
+
     Return_if_fail(make_subdirectories(filename));
 
     WriteableFile outfile;
@@ -1666,6 +1791,7 @@ void ProjectDescriptor::really_write_project_file (ostream& os)
 
 	    switch (string->type()) {
 	    case EsStringLiteral:
+
 		for (int i = 0; i < string->length(); i += 1) {
 		    char c = string->index(i);
 
@@ -1699,7 +1825,9 @@ void ProjectDescriptor::really_write_project_file (ostream& os)
 	} else /* (next_index == next_file_index) */ {
 	    FileEntry *fe = _file_entries->index (file_number++);
 
-	    os << "(" << fe->working_path() << " (";
+	    os << "(";
+	    print_protected(fe->working_path(), os);
+	    os << " (";
 
 	    if (fe->file_type() == SymLink && fe->link_name()) {
 		print_protected(fe->link_name(), os);
@@ -1709,9 +1837,18 @@ void ProjectDescriptor::really_write_project_file (ostream& os)
 		print_protected (project_name(), os);
 		os << "/";
 		print_protected (fe->descriptor_name(), os);
+#ifdef __GNUG__
 		os.form(" %s %03o",
 			fe->descriptor_version_number(),
 			fe->file_mode());
+#else
+                {
+                  char buf[8];
+                  sprintf(buf, "%03o", fe->file_mode());
+                  os << " " << fe->descriptor_version_number()
+                     << " " << buf;
+                }
+#endif
 	    }
 
 	    os << ")";
@@ -1958,6 +2095,13 @@ void Estring::append_protected (const char* string)
     protect_path (string, this);
 }
 
+void Estring::append_string (const char* string)
+{
+    append ("\"");
+    protect_string (string, this);
+    append ("\"");
+}
+
 void ProjectDescriptor::register_estring(Estring* estring, char missing)
 {
     if (!_all_estrings)
@@ -1994,7 +2138,12 @@ extern "C"
 int prjinput(void* buf, int max_size)
 {
     if (prj_input) {
-	return fread( buf, 1, max_size, prj_input);
+	int c;
+
+	If_fail (c << Err_fread( buf, max_size, prj_input))
+	  return -1;
+
+	return c;
     } else if (prj_seg) {
 	int length = prj_seg->length() - 1;
 
@@ -2027,7 +2176,7 @@ PrVoidError ProjectDescriptor::parse_prj_file(FILE* infile)
     prj_lex_this_index = 0;
 
     If_fail (prj_seg->map_file (fileno(infile)))
-	throw prcserror << "Failed reading from file " << squote (_prj_source_name) << perror;
+	pthrow prcserror << "Failed reading from file " << squote (_prj_source_name) << perror;
 
     _segment->append_segment("", 1); /* Added a null here for safety, going to
 				      * have to get rid of it when writing. */
@@ -2052,7 +2201,7 @@ PrVoidError ProjectDescriptor::parse_prj_file(FILE* infile)
 	case PrjEof:
 	    for (int i = 0; i < _prj_entries; i += 1)
 		if (_pftable[i].must_have && !_prj_entry_found[i])
-		    throw prcserror << "Missing project file entry "
+		    pthrow prcserror << "Missing project file entry "
 				    << squote(_pftable[i].name) << dotendl;
 
 	    _end_of_buffer_point = new Estring (this, prj_lex_cur_index);
@@ -2106,23 +2255,23 @@ PrVoidError ProjectDescriptor::read_parse_bad_token(PrjTokenType tok)
 {
     switch (tok) {
     case PrjEof:
-	throw prcserror << fileline(_prj_source_name, prj_lineno)
+	pthrow prcserror << fileline(_prj_source_name, prj_lineno)
 			<< "Unexpected EOF" << dotendl;
     case PrjNull:
-	throw prcserror << fileline(_prj_source_name, prj_lineno)
+	pthrow prcserror << fileline(_prj_source_name, prj_lineno)
 			<< "Illegal null character" << dotendl;
     case PrjBadString:
-	throw prcserror << fileline(_prj_source_name, prj_lineno)
+	pthrow prcserror << fileline(_prj_source_name, prj_lineno)
 			<< "Unterminated string constant" << dotendl;
     case PrjString:
     case PrjName:
-	throw prcserror << fileline(_prj_source_name, prj_lineno)
+	pthrow prcserror << fileline(_prj_source_name, prj_lineno)
 			<< "Unexpected token: " << squote(prj_lex_val) << dotendl;
     case PrjClose:
-	throw prcserror << fileline(_prj_source_name, prj_lineno)
+	pthrow prcserror << fileline(_prj_source_name, prj_lineno)
 			<< "Unexpected ')'" << dotendl;
     case PrjOpen:
-	throw prcserror << fileline(_prj_source_name, prj_lineno)
+	pthrow prcserror << fileline(_prj_source_name, prj_lineno)
 			<< "Unexpected '('" << dotendl;
     }
     return NoError;
@@ -2189,17 +2338,17 @@ PrVoidError ProjectDescriptor::read_list_checked(size_t offset)
     tok = prj_lex();
 
     if (tok != PrjName)
-	throw prcserror << fileline(_prj_source_name, prj_lineno)
+	pthrow prcserror << fileline(_prj_source_name, prj_lineno)
 			<< "Expecting label at head of list" << dotendl;
 
     pf = prj_lookup_func(prj_lex_val, prj_lex_len);
     h  = prj_lookup_hash(prj_lex_val, prj_lex_len);
 
     if (!pf)
-	throw prcserror << "Unexpected label " << squote (prj_lex_val) << dotendl;
+	pthrow prcserror << "Unexpected label " << squote (prj_lex_val) << dotendl;
 
     if (_prj_entry_found[h] && h != prj_lookup_hash ("Files", strlen ("Files")))
-	throw prcserror << fileline(_prj_source_name, prj_lineno)
+	pthrow prcserror << fileline(_prj_source_name, prj_lineno)
 			<< "Duplicate project file entry" << dotendl;
 
     _prj_entry_found[h] = true;
@@ -2208,6 +2357,7 @@ PrVoidError ProjectDescriptor::read_list_checked(size_t offset)
 	Return_if_fail (s << read_list_token (tok, offset));
 
 	/* I finally found a use for the ->* operator, fear it. */
+	/*Return_if_fail( (this ->* (pf->parse_func)) (s) );*/
 	Return_if_fail( (this ->* (pf->parse_func)) (s) );
 
 	delete s;
