@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: repository.cc 1.22.1.11.1.28.1.16.1.26 Thu, 09 Oct 1997 20:11:02 -0700 jmacd $
+ * $Id: repository.cc 1.22.1.11.1.28.1.16.1.33 Tue, 13 Oct 1998 13:28:35 -0700 jmacd $
  */
 
 
@@ -130,7 +130,7 @@ PrVoidError Rep_init_repository()
     }
 
     If_fail(Rep_guess_repository_path())
-	throw prcserror << "Please set your PRCS_REPOSITORY environment variable" << dotendl;
+	pthrow prcserror << "Please set your PRCS_REPOSITORY environment variable" << dotendl;
 
     rep_root_path->truncate(rep_root_path_len);
     Return_if_fail(Rep_init_command(rep_root_path->cast()));
@@ -142,10 +142,10 @@ static PrVoidError Rep_init_command(const char* path)
 {
     if (!fs_file_exists(path) ) {
 	if (option_report_actions)
-	    throw prcsinfo << "Create repository " << squote(path) << dotendl;
+	    pthrow prcsinfo << "Create repository " << squote(path) << dotendl;
 
 	If_fail( Err_mkdir(path, 0777 | S_ISGID | 01000))
-	    throw prcserror << "Cannot create repository " << squote(path) << perror;
+	    pthrow prcserror << "Cannot create repository " << squote(path) << perror;
 	else
 	    prcswarning << "Created repository " << squote(path)
 			<< ", you may wish to run " << squote("prcs admin access")
@@ -156,18 +156,18 @@ static PrVoidError Rep_init_command(const char* path)
 
     if(!fs_is_directory(rep_lock_name)) {
         if (option_report_actions)
-	    throw prcserror << "Create repository lock directory "
+	    pthrow prcserror << "Create repository lock directory "
 			    << squote(rep_lock_name) << perror;
 
 	Umask mask (0);
 
-	If_fail( Err_mkdir(rep_lock_name, 0777) )
-	    throw prcserror << "Cannot create repository lock directory "
+	If_fail( Err_mkdir(rep_lock_name, 0777 | S_ISGID) )
+	    pthrow prcserror << "Cannot create repository lock directory "
 			     << squote(rep_lock_name) << perror;
     }
 
     if(!fs_file_writeable(rep_lock_name))
-       throw prcserror << "You must have write permission on the lock directory "
+       pthrow prcserror << "You must have write permission on the lock directory "
 		       << squote(rep_lock_name) << perror;
 
     return NoError;
@@ -216,7 +216,7 @@ PrVoidError obtain_lock(AdvisoryLock* lock, bool write_lock)
 
     while(true) {
 	if(lockval == LOCK_FAIL)
-	    throw prcserror << "Failed obtaining a repository lock" << dotendl;
+	    pthrow prcserror << "Failed obtaining a repository lock" << dotendl;
 
 	if(lockval == LOCK_SUCCESS)
 	    return NoError;
@@ -228,7 +228,7 @@ PrVoidError obtain_lock(AdvisoryLock* lock, bool write_lock)
 	other_locks = lock->who_is_locking();
 
 	if(!other_locks)
-	    throw prcserror << "Could not obtain a list of lockers, unknown "
+	    pthrow prcserror << "Could not obtain a list of lockers, unknown "
 		"locking error" << dotendl;
 
 	prcsinfo << "Could not obtain a lock because of the following locks:" << prcsendl;
@@ -243,7 +243,7 @@ PrVoidError obtain_lock(AdvisoryLock* lock, bool write_lock)
 	fclose(other_locks);
 
 	if(lock_count == 0)
-	    throw prcserror << "Inconsistant lock information, cannot continue" << dotendl;
+	    pthrow prcserror << "Inconsistant lock information, cannot continue" << dotendl;
 
 	if(lockval == LOCK_TRY_STEAL) {
 	    prcsquery << "Lock age exceeds timeout age.  "
@@ -307,12 +307,12 @@ PrVoidError RepEntry::Rep_make_tag(const int created[3], const int usable[3])
     tag_file.close();
 
     if(tag_file.bad()) {
-	throw prcserror << "Failed writing repository tagfile "
+	pthrow prcserror << "Failed writing repository tagfile "
 			 << squote(file_name) << perror;
     }
 
     If_fail(Err_chmod(file_name, 0444)) {
- 	throw prcserror << "Chmod failed on new repository tagfile "
+ 	pthrow prcserror << "Chmod failed on new repository tagfile "
 			 << squote(file_name) << perror;
     }
 
@@ -327,18 +327,18 @@ PrVoidError RepEntry::Rep_verify_tag(UpgradeRepository* upgrades)
     int usable[3];
 
     if(!fs_file_exists(file_name)) {
-	throw prcserror << "Repository is not tagged, the file "
+	pthrow prcserror << "Repository is not tagged, the file "
 			 << squote(file_name) << " does not exist" << dotendl;
     }
 
     if(!fs_file_readable(file_name)) {
-	throw prcserror << "Can't read repository tag " << squote(file_name) << dotendl;
+	pthrow prcserror << "Can't read repository tag " << squote(file_name) << dotendl;
     }
 
     ifstream tag_file(file_name);
 
     if(tag_file.bad()) {
-	throw prcserror << "Can't open repository tag file " << squote(file_name) << dotendl;
+	pthrow prcserror << "Can't open repository tag file " << squote(file_name) << dotendl;
     }
 
     tag_file.get(buf, sizeof(prcs_created_by));
@@ -347,7 +347,7 @@ PrVoidError RepEntry::Rep_verify_tag(UpgradeRepository* upgrades)
     tag_file >> created[2];
 
     if(strncmp(buf, prcs_created_by, strlen(prcs_created_by)) != 0 || tag_file.bad()) {
-	throw prcserror << "Repository tag file "
+	pthrow prcserror << "Repository tag file "
 			 << squote(file_name) << " is invalid" << dotendl;
     }
 
@@ -358,14 +358,14 @@ PrVoidError RepEntry::Rep_verify_tag(UpgradeRepository* upgrades)
     tag_file >> usable[2];
 
     if(strncmp(buf, prcs_usable_by, strlen(prcs_usable_by)) != 0 || tag_file.bad()) {
-	throw prcserror << "Repository tag file " << squote(file_name) << " is invalid" << dotendl;
+	pthrow prcserror << "Repository tag file " << squote(file_name) << " is invalid" << dotendl;
     }
 
     bool confirm_upgrade = false;
     while(upgrades && upgrades->upgradeFunc) {
 	if(version_less_than(created, upgrades->version)) {
 	    if(!_writeable) {
-		throw prcserror << "This version of PRCS needs to modify your repository before "
+		pthrow prcserror << "This version of PRCS needs to modify your repository before "
 		    "continuing.  You do not currently have a write lock, required to do so.  "
 		    "You can run " << squote("prcs admin rebuild") << " to upgrade your repository" << dotendl;
 	    }
@@ -436,7 +436,7 @@ PrVoidError RepEntry::Rep_verify_tag(UpgradeRepository* upgrades)
      * to use, and the upgrade did not help, tell the user the
      * repository is too old to use.  */
     if(version_less_than(created, prcs_may_use_number)) {
-	throw prcserror << "This version of PRCS outdates the version which created your repository, "
+	pthrow prcserror << "This version of PRCS outdates the version which created your repository, "
 			 << created[0] << '.'
 			 << created[1] << '.'
 			 << created[2]
@@ -447,7 +447,7 @@ PrVoidError RepEntry::Rep_verify_tag(UpgradeRepository* upgrades)
      * use, print a message saying to upgrade the binary to the newer
      * PRCS.  */
     if(version_less_than(prcs_version_number, usable)) {
-	throw prcserror << "This version of PRCS is outdated by version "
+	pthrow prcserror << "This version of PRCS is outdated by version "
 			 << created[0] << '.'
 			 << created[1] << '.'
 			 << created[2] << ".  You must upgrade before using this repository"
@@ -478,10 +478,10 @@ PrVoidError RepEntry::Rep_create_repository_entry()
     const char* file_name = Rep_entry_path();
 
     if ( option_report_actions ) {
-	throw prcserror << "Create repository entry " << squote(_entry_name);
+	pthrow prcserror << "Create repository entry " << squote(_entry_name);
     } else {
-	If_fail( Err_mkdir(file_name, 0777) ) {
-	    throw prcserror << "Cannot create repository entry " << squote(_entry_name) << perror;
+	If_fail( Err_mkdir(file_name, 0777 | S_ISGID) ) {
+	    pthrow prcserror << "Cannot create repository entry " << squote(_entry_name) << perror;
 	}
 
 	Return_if_fail(Rep_make_tag(prcs_version_number, prcs_usable_number));
@@ -490,7 +490,7 @@ PrVoidError RepEntry::Rep_create_repository_entry()
 
 	If_fail(VC_register(NULL)) {
 	    fs_nuke_file(file_name);
-	    throw prcserror << "Failed registering project version file"
+	    pthrow prcserror << "Failed registering project version file"
 			    << squote(_project_file) << dotendl;
 	}
 
@@ -571,7 +571,7 @@ PrVoidError RepEntry::init(const char* entry, bool writeable0, bool create, bool
 	if(create) {
 	    Return_if_fail(Rep_create_repository_entry());
 	} else {
-	    throw prcserror << "Repository entry " << squote(entry)
+	    pthrow prcserror << "Repository entry " << squote(entry)
 			     << " does not exist, you can create an entry with "
 			     << squote("prcs checkin")
 			     << dotendl;
@@ -579,7 +579,7 @@ PrVoidError RepEntry::init(const char* entry, bool writeable0, bool create, bool
     }
 
     If_fail(Err_stat(Rep_entry_path(), &_rep_stat_buf))
-	throw prcserror << "Stat failed on repository entry "
+	pthrow prcserror << "Stat failed on repository entry "
 			 << squote(Rep_entry_path()) << perror;
 
     _rep_mask = (_rep_stat_buf.st_mode & 0x1ff) ^ 0777;
@@ -590,16 +590,16 @@ PrVoidError RepEntry::init(const char* entry, bool writeable0, bool create, bool
     install_alarm_handler  (repository_alarm_handler, this);
 
     if(_writeable && !(fs_file_rwx(Rep_entry_path()))) {
-	throw prcserror << "You need read, write, and execute permission on the repository entry "
+	pthrow prcserror << "You need read, write, and execute permission on the repository entry "
 			 << squote(Rep_entry_path()) << dotendl;
     } else if(!fs_file_executable(Rep_entry_path())) {
-	throw prcserror << "You need execute permission on the repository entry "
+	pthrow prcserror << "You need execute permission on the repository entry "
 			 << squote(Rep_entry_path()) << dotendl;
     }
 
     if (require_db) {
 	if(!fs_file_exists(Rep_name_in_entry(prcs_data_file_name))) {
-	    throw prcserror << "Repository entry " << squote(entry)
+	    pthrow prcserror << "Repository entry " << squote(entry)
 			    << " is missing its data file, please run "
 			    << squote("prcs admin rebuild")
 			    << " to generate this file" << dotendl;
@@ -614,14 +614,19 @@ PrVoidError RepEntry::init(const char* entry, bool writeable0, bool create, bool
 	_rcs_file_table = _rebuild_file->get_rcs_file_summary();
     }
 
-    _rfl = new RepFreeFilename(Rep_entry_path());
+    _rfl = new RepFreeFilename(Rep_entry_path(), _rep_mask);
 
     _compressed = fs_file_exists(Rep_name_in_entry(prcs_compression_tag));
 
+#ifndef __GNUG__
+#  define APPENDTAG ios::out|ios::app
+#else
+#  define APPENDTAG "a"
+#endif
     if (_writeable) {
 	_log_stream = new filebuf;
-	if (!_log_stream->open(Rep_name_in_entry("prcs_log"), "a"))
-	    throw prcserror << "Failed opening repository log file" << perror;
+	if (!_log_stream->open(Rep_name_in_entry("prcs_log"), APPENDTAG))
+	    pthrow prcserror << "Failed opening repository log file" << perror;
 	_log_pstream = new PrettyStreambuf(_log_stream, NULL);
 	_log_pstream->set_fill_width (1<<20);
 	_log_pstream->set_fill_prefix ("log: ");
@@ -642,7 +647,7 @@ static PrVoidError uncompress_file(const char* name, const void* data)
 
     if (len > 4 && strcmp(".gz", basename + len - 3) == 0) {
 	if (option_report_actions)
-	    throw prcserror << "You may not perform this command on a compressed repository with -n" << dotendl;
+	    pthrow prcserror << "You may not perform this command on a compressed repository with -n" << dotendl;
 
 	if(once) {
 	    prcsinfo << "Uncompressing repository entry "
@@ -657,7 +662,7 @@ static PrVoidError uncompress_file(const char* name, const void* data)
 	args->append(name);
 
 	Return_if_fail_if_ne(gzip_command.open_stdout(), 0) {
-	    throw prcserror << "Gzip returned non-zero status on file "
+	    pthrow prcserror << "Gzip returned non-zero status on file "
 			    << squote (name)
 			    << ", aborting" << dotendl;
 	}
@@ -679,7 +684,7 @@ static PrVoidError compress_file(const char* name, const void* data)
     if(len > 3 && strcmp(",v", basename + len - 2) == 0 &&
        strcmp(basename, prjname) != 0) {
 	if (option_report_actions)
-	    throw prcserror << "You may not perform this command on a compressed repository with -n" << dotendl;
+	    pthrow prcserror << "You may not perform this command on a compressed repository with -n" << dotendl;
 
 	if(once) {
 	    prcsinfo << "Compressing repository entry "
@@ -693,7 +698,7 @@ static PrVoidError compress_file(const char* name, const void* data)
 	args->append(name);
 
 	Return_if_fail_if_ne(gzip_command.open_stdout(), 0) {
-	    throw prcserror << "Gzip returned non-zero status on file "
+	    pthrow prcserror << "Gzip returned non-zero status on file "
 			    << squote (name)
 			    << ", aborting" << dotendl;
 	}
@@ -756,7 +761,7 @@ PrConstCharPtrError RepEntry::Rep_prepare_file(const char* file)
 	    args->append(zipname);
 
 	    Return_if_fail_if_ne(gzip_command.open_stdout(), 0)
-		throw prcserror << "Gzip returned non-zero status, aborting" << dotendl;
+		pthrow prcserror << "Gzip returned non-zero status, aborting" << dotendl;
 	} else {
 	    const char* temp = make_temp_file(",v");
 
@@ -770,7 +775,7 @@ PrConstCharPtrError RepEntry::Rep_prepare_file(const char* file)
 	    Return_if_fail(fs_write_filename(gzip_command.standard_out(), temp));
 
 	    Return_if_fail_if_ne(gzip_command.close(), 0)
-		throw prcserror << "Gzip returned non-zero status, aborting" << dotendl;
+		pthrow prcserror << "Gzip returned non-zero status, aborting" << dotendl;
 
 	    _rep_comp_cache->append (temp); /* this memory is ours to
 					     * free, except that it's
@@ -788,7 +793,7 @@ PrConstCharPtrError RepEntry::Rep_prepare_file(const char* file)
     }
 
     if(!fs_file_exists(fullname))
-	throw prcserror << "Repository file " << squote(fullname)
+	pthrow prcserror << "Repository file " << squote(fullname)
 			 << " not found--try running " << squote("prcs admin rebuild")
 			 << " for a complete diagnosis" << dotendl;
 
@@ -842,7 +847,7 @@ static PrVoidError copy_change_ownership(const char* name)
     Return_if_fail(fs_copy_filename(name, temp));
 
     If_fail(Err_rename(temp, name))
-	throw prcserror << "Rename " << squote(temp) << " to " << squote(name)
+	pthrow prcserror << "Rename " << squote(temp) << " to " << squote(name)
 			<< " failed while changing group -- correct this manually!" << perror;
 
     return NoError;
@@ -854,7 +859,7 @@ static PrVoidError set_ugrid(const char* name, uid_t uid, gid_t gid, struct stat
 	Return_if_fail(copy_change_ownership(name));
 
     If_fail (Err_chown(name, uid, gid))
-	throw prcserror << "Chown failed on " << squote(name) << perror;
+	pthrow prcserror << "Chown failed on " << squote(name) << perror;
 
     return NoError;
 }
@@ -870,12 +875,12 @@ PrVoidError Rep_chown_dir_callback (const char* name, const void* v_data)
     RepChownCallbackData* data = (RepChownCallbackData*) v_data;
 
     If_fail (Err_stat(name, &buf))
-	throw prcserror << "Stat failed on " << squote(name) << perror;
+	pthrow prcserror << "Stat failed on " << squote(name) << perror;
 
     if (buf.st_uid != data->uid || buf.st_gid != data->gid) {
 	/* No copying here, only the owner or root can do it. */
 	If_fail (Err_chown(name, data->uid, data->gid))
-	    throw prcserror << "Chown failed on " << squote(name) << perror;
+	    pthrow prcserror << "Chown failed on " << squote(name) << perror;
     }
 
     return NoError;
@@ -887,7 +892,7 @@ PrVoidError Rep_chown_callback (const char* name, const void* v_data)
     RepChownCallbackData* data = (RepChownCallbackData*) v_data;
 
     If_fail (Err_stat(name, &buf))
-	throw prcserror << "Stat failed on " << squote(name) << perror;
+	pthrow prcserror << "Stat failed on " << squote(name) << perror;
 
     if (buf.st_uid != data->uid || buf.st_gid != data->gid)
 	Return_if_fail(set_ugrid(name, data->uid, data->gid, &buf));
@@ -902,7 +907,7 @@ PrVoidError RepEntry::Rep_chown_file (const char* desc)
     full_name.append (",v");
 
     If_fail(Err_chmod (full_name, (0777 ^ _rep_mask) & 0444))
-	throw prcserror << "Chown failed on " << full_name << perror;
+	pthrow prcserror << "Chown failed on " << full_name << perror;
 
     return NoError;
 }
@@ -916,12 +921,12 @@ PrVoidError RepEntry::Rep_chown (const char* user, const char* group)
     grp = getgrnam(group);
 
     if (!grp)
-        throw prcserror << "Cannot lookup group " << squote(group) << perror;
+        pthrow prcserror << "Cannot lookup group " << squote(group) << perror;
 
     pwd = getpwnam(user);
 
     if (!pwd)
-        throw prcserror << "Cannot lookup user " << squote(user) << perror;
+        pthrow prcserror << "Cannot lookup user " << squote(user) << perror;
 
     data.gid = grp->gr_gid;
     data.uid = pwd->pw_uid;
@@ -929,7 +934,7 @@ PrVoidError RepEntry::Rep_chown (const char* user, const char* group)
     const char* dir_name = Rep_name_in_repository(_entry_name);
 
     If_fail (Err_chown(dir_name, data.uid, data.gid))
-	throw prcserror << "Chown failed on " << squote(dir_name) << perror;
+	pthrow prcserror << "Chown failed on " << squote(dir_name) << perror;
 
     Return_if_fail (directory_recurse_dirs (dir_name, &data, Rep_chown_dir_callback));
 
@@ -942,7 +947,7 @@ PrVoidError RepEntry::Rep_chown (const char* user, const char* group)
         const char* file = (f);                                    \
 	const mode_t m = (m2);                                     \
 	If_fail(Err_chmod(file, m))                                \
-	    throw prcserror << "Chmod failed on repository entry " \
+	    pthrow prcserror << "Chmod failed on repository entry " \
 			    << squote(file) << perror;             \
 	} while (false)
 
@@ -1080,10 +1085,13 @@ PrVoidError RepFreeFilename::real_open_dir (const char* dirname)
 
 	    p += 2;
 
-	    if (!fs_is_directory(fullname_x))
-		If_fail(Err_mkdir(fullname_x, 0777))
-		    throw prcserror << "Cannot create repository subdirectory "
+	    if (!fs_is_directory(fullname_x)) {
+	        Umask mask (_umask);
+
+		If_fail(Err_mkdir(fullname_x, 0777 | S_ISGID))
+		    pthrow prcserror << "Cannot create repository subdirectory "
 				    << squote(fullname_x) << perror;
+	    }
 	} while (*p);
     }
 
@@ -1105,7 +1113,7 @@ PrVoidError RepFreeFilename::real_open_dir (const char* dirname)
     }
 
     if (! dir.OK() )
-	throw prcserror << "Error reading repository directory " << squote(fullname)
+	pthrow prcserror << "Error reading repository directory " << squote(fullname)
 			<< perror;
 
     return NoError;
@@ -1306,7 +1314,7 @@ PrRcsVersionDataPtrError RepEntry::lookup_rcs_file_data(const char* filename,
 	    return data;
     }
 
-    throw prcserror << "Repository inconsistency detected in repository file "
+    pthrow prcserror << "Repository inconsistency detected in repository file "
 		     << squote(filename) << " at file version " << version_num
 		     << ", please run " << squote("prcs admin rebuild")
 		     << " to diagnose this error" << dotendl;
@@ -1315,7 +1323,11 @@ PrRcsVersionDataPtrError RepEntry::lookup_rcs_file_data(const char* filename,
 RepEntry::~RepEntry()
 {
     if(_log) {
+#ifdef __GNUG__
 	if (_log_pstream->sync())
+#else
+        if (_log_pstream->pubsync())
+#endif
 	    prcserror << "warning: Write to repository log failed" << perror;
 
 	delete _log;
@@ -1370,10 +1382,12 @@ PrProjectDescriptorPtrError RepEntry::checkout_prj_file(const char* fullname,
 
     Return_if_fail(cofile << VC_checkout_stream(version, Rep_name_of_version_file()));
 
-    Return_if_fail(project << read_project_file(fullname, filename, false, cofile, flags));
+    PrVoidError it = (project << read_project_file(fullname, filename, false, cofile, flags));
 
     Return_if_fail(VC_close_checkout_stream(cofile, version, Rep_name_of_version_file()));
 
+    Return_if_fail (it);
+
     project->repository_entry(this);
 
     return project;
@@ -1395,7 +1409,7 @@ PrProjectDescriptorPtrError RepEntry::checkout_create_prj_file (const char *file
 
 PrVoidError RepEntry::Rep_alarm() const
 {
-    if (_repository_lock->touch_lock() != LOCK_SUCCESS)
+    if (_repository_lock && _repository_lock->touch_lock() != LOCK_SUCCESS)
 	prcsinfo << "Failed refreshing repository lock, something bad "
 	    "is happening" << dotendl;
 
@@ -1442,15 +1456,15 @@ PrIntError RepEntry::Rep_print_access()
     pwd = getpwuid(_rep_stat_buf.st_uid);
 
     if(!pwd)
-	throw prcserror << "Can't lookup repository owner's name (getpwuid)" << dotendl;
+	pthrow prcserror << "Can't lookup repository owner's name (getpwuid)" << dotendl;
 
     grp = getgrgid(_rep_stat_buf.st_gid);
 
     if(!grp)
-	throw prcserror << "Can't lookup repository group owner's name (getgrgid)" << dotendl;
+	pthrow prcserror << "Can't lookup repository group owner's name (getgrgid)" << dotendl;
 
-    _owner_name = p_strdup(pwd->pw_name);
-    _group_name = p_strdup(grp->gr_name);
+    _owner_name = p_strdup(pwd->pw_name ? pwd->pw_name : "nobody");
+    _group_name = p_strdup(grp->gr_name ? grp->gr_name : "nogroup");
 
     print_access(&_rep_stat_buf, pwd, grp);
 
@@ -1464,7 +1478,7 @@ static PrVoidError can_set_group(const char* group_name, const char *current_gro
     if(strcmp(group_name, current_group_name) != 0) {
 
 	if((grp = getgrnam(group_name)) == NULL)
-	    throw prcserror << "Can't lookup group " << squote(group_name) << dotendl;
+	    pthrow prcserror << "Can't lookup group " << squote(group_name) << dotendl;
 
 	bool found = false;
 	char** grp_members = grp->gr_mem;
@@ -1480,7 +1494,7 @@ static PrVoidError can_set_group(const char* group_name, const char *current_gro
 	}
 
 	if(!found)
-	    throw prcserror << "You are not a member of that group" << dotendl;
+	    pthrow prcserror << "You are not a member of that group" << dotendl;
     }
 
     return NoError;
@@ -1564,8 +1578,9 @@ const char* RepEntry::Rep_entry_path()
 }
 
 
-RepFreeFilename::RepFreeFilename(const char* name)
+RepFreeFilename::RepFreeFilename(const char* name, int umask)
 {
+    _umask = umask;
     _directory = 0;
     _root.assign (name);
     real_open_dir ("");
@@ -1613,7 +1628,7 @@ PrPrcsExitStatusError admin_compress_command()
 	Umask mask (rep_entry->Rep_get_umask());
 
 	if(creat(rep_entry->Rep_name_in_entry(prcs_compression_tag), 0444) < 0)
-	    throw prcserror << "Failed writing repository tagfile" << perror;
+	    pthrow prcserror << "Failed writing repository tagfile" << perror;
 
 	rep_entry->Rep_set_compression(true);
     }
@@ -1640,7 +1655,7 @@ PrPrcsExitStatusError admin_uncompress_command()
     }
 
     If_fail(Err_unlink(rep_entry->Rep_name_in_entry(prcs_compression_tag))) {
-	throw prcserror << "Failed unlinking repository compression tag " << perror;
+	pthrow prcserror << "Failed unlinking repository compression tag " << perror;
     }
 
     if(option_immediate_uncompression) {
@@ -1674,7 +1689,7 @@ PrPrcsExitStatusError admin_access_command()
 	Return_if_fail(old_mode << rep_entry->Rep_print_access());
 
 	if (!fs_same_file_owner(rep_entry->Rep_entry_path()) && uid != 0)
-	    throw prcserror <<"Only the owner of this repository entry may modify its "
+	    pthrow prcserror <<"Only the owner of this repository entry may modify its "
 		"access permissions" << dotendl;
 
 	if (uid == 0 || true) {
@@ -1719,25 +1734,25 @@ PrPrcsExitStatusError admin_access_command()
 	struct stat stat_buf;
 
 	If_fail(Err_stat(Rep_name_in_repository(""), &stat_buf))
-	    throw prcserror << "Stat failed on " << squote(rep_name) << perror;
+	    pthrow prcserror << "Stat failed on " << squote(rep_name) << perror;
 
 	pwd = getpwuid(stat_buf.st_uid);
 
 	if(!pwd)
-	    throw prcserror << "Can't lookup repository owner's name (getpwuid)" << dotendl;
+	    pthrow prcserror << "Can't lookup repository owner's name (getpwuid)" << dotendl;
 
 	grp = getgrgid(stat_buf.st_gid);
 
 	if(!grp)
-	    throw prcserror << "Can't lookup repository group owner's name (getgrgid)" << dotendl;
+	    pthrow prcserror << "Can't lookup repository group owner's name (getgrgid)" << dotendl;
 
-	owner_name = p_strdup(pwd->pw_name);
-	group_name = p_strdup(grp->gr_name);
+	owner_name = p_strdup(pwd->pw_name ? pwd->pw_name : "nobody");
+	group_name = p_strdup(grp->gr_name ? grp->gr_name : "nogroup");
 
 	print_access(&stat_buf, pwd, grp);
 
 	if(!fs_same_file_owner(rep_name) && uid != 0) {
-	    throw prcserror <<"Only the owner of this repository may modify its "
+	    pthrow prcserror <<"Only the owner of this repository may modify its "
 		"access permissions" << dotendl;
 	}
 
@@ -1765,12 +1780,12 @@ PrPrcsExitStatusError admin_access_command()
 	    return ExitSuccess;
 
 	If_fail(Err_chmod(rep_name, (0777 ^ mask) | S_ISGID | 01000))
-	    throw prcserror << "Chmod failed on " << squote(rep_name) << perror;
+	    pthrow prcserror << "Chmod failed on " << squote(rep_name) << perror;
 
 	const char* rep_lock_name = Rep_name_in_repository(prcs_lock_dir);
 
 	If_fail(Err_chmod(rep_lock_name, 0777 ^ mask))
-	    throw prcserror << "Chmod failed on " << squote(rep_lock_name) << perror;
+	    pthrow prcserror << "Chmod failed on " << squote(rep_lock_name) << perror;
 
     }
 
@@ -1800,7 +1815,7 @@ static PrVoidError repository_cleanup_handler(void* data, bool signaled)
 	    prcswarning << "Repository entry " << rep_entry->Rep_entry_path()
 			<< " not recompressed" << dotendl;
 	}
-    } else {
+    } else if (! option_report_actions) {
 	rep_entry->Rep_compress_all_files();
     }
 
